@@ -22,7 +22,6 @@ test_that("defensive", {
     expect_error(write_ods(c(123), ods_path, sheet = "whatever1", append = TRUE))
     expect_error(write_fods(c(22323), fods_path, sheet = "whatever1", append = TRUE))
     expect_error(write_fods(list(c(22323)), fods_path, sheet = "whatever1", append = TRUE))
-
 })
 
 .test_funcs <- function(funcs) {
@@ -30,52 +29,52 @@ test_that("defensive", {
     ## If you write a dataframe which has not had rownames explicitly set and use row_names=T,
     ## reading it back and comparing will give an attribute difference
     starwars10 <- readRDS("../testdata/starwars10.rds")
-    expect_silent(tmp <- funcs[["w"]](starwars10, path = tempfile(fileext = ".ods"), sheet = "SW", row_names = FALSE, col_names = FALSE))
-    expect_true(file.exists(tmp))
-    expect_silent(funcs[["w"]](starwars10, tmp, "SWR", row_names = TRUE, col_names = FALSE, append = TRUE))
-    expect_silent(funcs[["w"]](starwars10, tmp, "SWC", row_names = FALSE, col_names = TRUE, append = TRUE))
-    expect_silent(funcs[["w"]](starwars10, tmp, "SWRC", row_names = TRUE, col_names = TRUE, append = TRUE))
-    expect_silent(funcs[["w"]](starwars10[1, seq_len(ncol(starwars10))], tmp, "SW1", row_names = TRUE, col_names = TRUE, append = TRUE))
-    expect_silent(funcs[["w"]](starwars10[seq_len(nrow(starwars10)), 1, drop = FALSE], tmp, "SW10", row_names=TRUE, col_names = TRUE, append = TRUE))
+    withr::with_tempfile("tempods", fileext = ".ods", code = {
+        expect_silent(funcs[["w"]](starwars10, path = tempods, sheet = "SW", row_names = FALSE, col_names = FALSE))
+        expect_true(file.exists(tempods))
+        expect_silent(funcs[["w"]](starwars10, tempods, "SWR", row_names = TRUE, col_names = FALSE, append = TRUE))
+        expect_silent(funcs[["w"]](starwars10, tempods, "SWC", row_names = FALSE, col_names = TRUE, append = TRUE))
+        expect_silent(funcs[["w"]](starwars10, tempods, "SWRC", row_names = TRUE, col_names = TRUE, append = TRUE))
+        expect_silent(funcs[["w"]](starwars10[1, seq_len(ncol(starwars10))], tempods, "SW1", row_names = TRUE, col_names = TRUE, append = TRUE))
+        expect_silent(funcs[["w"]](starwars10[seq_len(nrow(starwars10)), 1, drop = FALSE], tempods, "SW10", row_names=TRUE, col_names = TRUE, append = TRUE))
 
-    ## SWRC is there
-    expect_error(funcs[["w"]](starwars10, tmp, "SWRC", row_names = TRUE, col_names = TRUE, append = TRUE))
-    ## SWRC is there, but this is update
-    expect_error(funcs[["w"]](starwars10, tmp, "SWRC", row_names = TRUE, col_names = TRUE, update = TRUE), NA)
-    expect_error(funcs[["w"]](starwars10, tmp, "whatevernotexists", row_names = TRUE, col_names = TRUE, update = TRUE))
+        ## SWRC is there
+        expect_error(funcs[["w"]](starwars10, tempods, "SWRC", row_names = TRUE, col_names = TRUE, append = TRUE))
+        ## SWRC is there, but this is update
+        expect_error(funcs[["w"]](starwars10, tempods, "SWRC", row_names = TRUE, col_names = TRUE, update = TRUE), NA)
+        expect_error(funcs[["w"]](starwars10, tempods, "whatevernotexists", row_names = TRUE, col_names = TRUE, update = TRUE))
 
-    df <- suppressMessages(funcs[["r"]](tmp, "SW", row_names = FALSE, col_names = FALSE, strings_as_factors = TRUE, as_tibble = FALSE))
-    expect_true(all.equal({
-        cars <- starwars10
-        rownames(cars) <- NULL
-        colnames(cars) <- vctrs::vec_as_names(rep("", 9), repair = "unique")
-        cars
-    }, df))
+        df <- suppressMessages(funcs[["r"]](tempods, "SW", row_names = FALSE, col_names = FALSE, strings_as_factors = TRUE, as_tibble = FALSE))
+        expect_true(all.equal({
+            cars <- starwars10
+            rownames(cars) <- NULL
+            colnames(cars) <- vctrs::vec_as_names(rep("", 9), repair = "unique")
+            cars
+        }, df))
 
-    df <- funcs[["r"]](tmp, "SWR", row_names = TRUE, col_names = FALSE, strings_as_factors = TRUE, as_tibble = FALSE)
-    expect_true(all.equal({
-        cars <- starwars10
-        colnames(cars) <- vctrs::vec_as_names(rep("", 9), repair = "unique")
-        cars}, df))
+        df <- funcs[["r"]](tempods, "SWR", row_names = TRUE, col_names = FALSE, strings_as_factors = TRUE, as_tibble = FALSE)
+        expect_true(all.equal({
+            cars <- starwars10
+            colnames(cars) <- vctrs::vec_as_names(rep("", 9), repair = "unique")
+            cars}, df))
 
-    df <- funcs[["r"]](tmp, "SWC", row_names = FALSE, col_names = TRUE, strings_as_factors = TRUE, as_tibble = FALSE)
-    expect_true(all.equal({
-        cars <- starwars10
-        rownames(cars) <- NULL
-        cars
-    }, df))
+        df <- funcs[["r"]](tempods, "SWC", row_names = FALSE, col_names = TRUE, strings_as_factors = TRUE, as_tibble = FALSE)
+        expect_true(all.equal({
+            cars <- starwars10
+            rownames(cars) <- NULL
+            cars
+        }, df))
 
-    df <- funcs[["r"]](tmp, "SWRC", row_names = TRUE, col_names = TRUE, strings_as_factors = TRUE, as_tibble = FALSE)
-    expect_true(all.equal(starwars10, df))
+        df <- funcs[["r"]](tempods, "SWRC", row_names = TRUE, col_names = TRUE, strings_as_factors = TRUE, as_tibble = FALSE)
+        expect_true(all.equal(starwars10, df))
 
-    df <- funcs[["r"]](tmp, "SW1", row_names = TRUE, col_names = TRUE, strings_as_factors = TRUE, as_tibble = FALSE)
-
-    expect_false(isTRUE(all.equal(starwars10[1, seq_len(ncol(starwars10))], df))) # factor mismatch
-    expect_true(all((df == starwars10[1, seq_len(ncol(starwars10))])[1,]))
-
-    df <- funcs[["r"]](tmp, "SW10", row_names = TRUE, col_names = TRUE, strings_as_factors = TRUE, as_tibble = FALSE)
-    expect_true(all.equal(starwars10[seq_len(nrow(starwars10)), 1, drop = FALSE], df))
+        df <- funcs[["r"]](tempods, "SW1", row_names = TRUE, col_names = TRUE, strings_as_factors = TRUE, as_tibble = FALSE)
 
+        expect_false(isTRUE(all.equal(starwars10[1, seq_len(ncol(starwars10))], df))) # factor mismatch
+        expect_true(all((df == starwars10[1, seq_len(ncol(starwars10))])[1,]))
+        df <- funcs[["r"]](tempods, "SW10", row_names = TRUE, col_names = TRUE, strings_as_factors = TRUE, as_tibble = FALSE)
+        expect_true(all.equal(starwars10[seq_len(nrow(starwars10)), 1, drop = FALSE], df))
+    })
 }
 
 test_that("Update / append sheets ods & fods", {
@@ -85,10 +84,11 @@ test_that("Update / append sheets ods & fods", {
 
 test_that("issue 107", {
     legend <- readRDS("../testdata/legend.rds")
-    temp <- tempfile()
-    expect_error(write_ods(legend, path = temp, sheet = "Legend"), NA)
-    expect_error(write_ods(legend, path = temp, sheet = "Legend", update = TRUE), NA)
-    expect_error(write_ods(legend, path = temp, sheet = "Legend2", append = TRUE), NA)
+    withr::with_tempfile("temp", {
+        expect_error(write_ods(legend, path = temp, sheet = "Legend"), NA)
+        expect_error(write_ods(legend, path = temp, sheet = "Legend", update = TRUE), NA)
+        expect_error(write_ods(legend, path = temp, sheet = "Legend2", append = TRUE), NA)
+    })
 })
 
 test_empty_edge <- function(.write_func, .list_func, .read_func) {
@@ -117,8 +117,9 @@ test_that("edge cases", {
     test_empty_edge(write_ods, list_ods_sheets, read_ods)
     test_empty_edge(write_fods, list_fods_sheets, read_fods)
     ## #163
-    rubbish_file <- tempfile(fileext = ".fods")
-    writeLines("<ul><li>hello</li></ul>", rubbish_file)
-    expect_error(write_fods(mtcars, rubbish_file, sheet = "whatever", append = TRUE))
-    expect_error(write_fods(mtcars, rubbish_file, sheet = "whatever", update = TRUE))
+    withr::with_tempfile("rubbish_file", {
+        writeLines("<ul><li>hello</li></ul>", rubbish_file)
+        expect_error(write_fods(mtcars, rubbish_file, sheet = "whatever", append = TRUE))
+        expect_error(write_fods(mtcars, rubbish_file, sheet = "whatever", update = TRUE))
+    }, fileext = ".fods")
 })
